@@ -11,22 +11,51 @@ It is composed of multiple directives and settings:
 
 
 """
+import json
 import os
+import re
 
+from dataclasses import dataclass, field
 from itertools import groupby
 
-from typing import cast, Any, List, Optional, Tuple
+from typing import Any, List, Optional, Tuple, Dict
 
 from docutils.statemachine import ViewList, StringList
 from docutils import nodes
+from sphinx.domains.c import CObject
 from sphinx.application import Sphinx
-from sphinx.directives import SphinxDirective
 from sphinx.util import logging
 from sphinx.util.docstrings import prepare_docstring
 from sphinx.ext.autodoc import Documenter, members_option, bool_option
 from sphinx.ext.autodoc.directive import DocumenterBridge
 
 from sphinx_c_autodoc import loader
+from sphinx_c_autodoc.domains.c import patch_c_domain
+
+
+# TODO not real fond of this being here in the main c autodoc file, need to
+# find a way to make it easier to cache the documented files.
+@dataclass
+class ViewCodeListing:
+    """
+    A data structure used for constructing a viewcode source listing.
+
+    Attributes:
+        raw_listing:
+            The plain text representation of the code. This should be
+            basically the output of file.read().
+
+        ast (Dict):
+            A dictionary like representation of the code constructs.
+            See :ref:`developer_notes:Common Terms`.
+
+        doc_links (Dict): To be used by the consumers, i.e. viewcode.
+
+    """
+
+    raw_listing: str
+    ast: Dict
+    doc_links: Dict = field(default_factory=dict)
 
 
 logger = logging.getLogger(__name__)
@@ -49,7 +78,11 @@ class CObjectDocumenter(Documenter):
     # objects
     priority = 11
 
-    option_spec = {"members": members_option, "noindex": bool_option}
+    option_spec = {
+        "members": members_option,
+        "noindex": bool_option,
+        "private-members": bool_option,
+    }
 
     @classmethod
     def can_document_member(
@@ -78,8 +111,44 @@ class CObjectDocumenter(Documenter):
             isinstance(parent, CObjectDocumenter) and member.type == cls.directivetype
         )
 
+    def parse_name(self) -> bool:
+        """Determine what module to import and what attribute to document.
+
+        .. note:: Sphinx autodoc supports args and return anotation, since
+            this is targeting C and it isn't currently needed, these won't be
+            supported by this implementation.
+
+        Returns:
+            bool: True if successfully parsed and sets :attr:`modname`,
+                :attr:`objpath`, :attr:`fullname`.
+
+                False if the signature couldn't be parsed.
+        """
+        c_autodoc_re = re.compile(r"^([\w\/\\.]+)(::([\w.]+\.)?(\w+))?\s*$")
+
+        try:
+            match = c_autodoc_re.match(self.name)
+            fullname, _, path, base = match.groups()  # type: ignore
+        except AttributeError:
+            logger.warning(
+                "invalid signature for auto%s (%r)" % (self.objtype, self.name),
+                type="c_autodoc",
+            )
+            return False
+
+        parents: List[str]
+        if path is None:
+            parents = []
+        else:
+            parents = path.rstrip(".").split(".")
+
+        self.modname, self.objpath = self.resolve_name(fullname, parents, path, base)
+
+        self.fullname = self.modname
+        return True
+
     def resolve_name(
-        self, modname: Optional[str], parents: List[str], path: Optional[str], base: str
+        self, modname: str, parents: List[str], path: Optional[str], base: str
     ) -> Tuple[str, List[str]]:
         """
         Resolve the module and object name of the object to document.
@@ -90,8 +159,7 @@ class CObjectDocumenter(Documenter):
           `my_file.c::some_func`.
 
         Args:
-            modname (str): Only set when called with double colons.  This will
-                be the left side of the double colons.
+            modname (str): The filename of the c file (module)
             parents (list): The list split('.') version of path.
 
                 - The filename without the extension when naked argument is used.
@@ -104,20 +172,16 @@ class CObjectDocumenter(Documenter):
                 - None if `parents` is the empty list.
                 - The ``'.'.join()`` version of `parents`, with a trailing ``.``.
 
-            base (str): The name of the object:
-
-                - This will be the file extension when naked argument is used.
-                - This will be the object, function, type, etc when double colon
-                  argument is used.
+            base (str): The name of the object to document. This will be None
+                when the object to document is the c module
 
         Returns:
             tuple: (str, [str]) The module name, and the object names (if any).
-                The object names will be joined with a `.`.
         """
-        if modname:
+        if base:
             return modname, parents + [base]
 
-        return cast(str, path) + base, []
+        return modname, []
 
     def import_object(self) -> bool:
         """Load the C file and build up the document structure.
@@ -126,37 +190,44 @@ class CObjectDocumenter(Documenter):
         """
         for source_dir in self.env.config.c_autodoc_roots:
             filename = os.path.join(source_dir, self.get_real_modname())
-            rel_filename, filename = self.env.relfn2path(filename)
+
+            # Prefixing with "/" will force "absolute" path which is relative
+            # to the source directory.
+            rel_filename, filename = self.env.relfn2path(f"/{filename}")
             if os.path.isfile(filename):
                 break
         else:
             logger.warning(
                 "Unable to find file, %s, in any of the directories %s "
-                "all directories are relative to the sphinx configuration "
-                "file" % (self.get_real_modname(), self.env.config.c_autodoc_roots),
+                "all directories are relative to the top documentation source directory"
+                % (self.get_real_modname(), self.env.config.c_autodoc_roots),
                 location=(self.env.docname, self.directive.lineno),
             )
             return False
 
         self.env.note_dependency(rel_filename)
 
+        source_dict = getattr(self.env, "_viewcode_c_modules", {})
+        self.env._viewcode_c_modules = source_dict  # type: ignore
+
         # TODO The :attr:`temp_data` is reset for each document ideally want to
         # use or make an attribute on `self.env` that is reset per run or just
         # not pickled.
-        if "c:loaded_modules" not in self.env.temp_data:
-            self.env.temp_data["c:loaded_modules"] = {}
+        modules_dict = self.env.temp_data.setdefault("c:loaded_modules", {})
 
-        if filename not in self.env.temp_data["c:loaded_modules"]:
+        if filename not in modules_dict:
             with open(filename) as f:
                 contents = [f.read()]
 
             # let extensions preprocess files
             self.env.app.emit("c-autodoc-pre-process", filename, contents)
-            self.env.temp_data["c:loaded_modules"][filename] = loader.load(
-                filename, contents[0]
+            modules_dict[filename] = loader.load(filename, contents[0])
+            ast = json.loads(str(modules_dict[filename]))
+            source_dict.setdefault(
+                self.get_real_modname(), ViewCodeListing(contents[0], ast)
             )
 
-        self.module = self.env.temp_data["c:loaded_modules"][filename]
+        self.module = modules_dict[filename]
 
         self.object = self.module
         self.object_name = self.name
@@ -209,22 +280,30 @@ class CObjectDocumenter(Documenter):
     ) -> List[Tuple[str, Any, bool]]:
         """Filter the given member list.
 
+        :meth:`filter_members` is called *after* :meth:`get_object_members`,
+        this means if `want_all` is False then only private members which
+        were explicitly requested will be in this list. Only when `want_all`
+        is True do we need to actually condition on private member.
+
         Members are skipped if
 
         - they are private (except if given explicitly or the private-members
           option is set)
-        - they are special methods (except if given explicitly or the
-          special-members option is set)
         - they are undocumented (except if the undoc-members option is set)
+          TODO not implemented yet.
 
+        TODO not implemented yet.
         The user can override the skipping decision by connecting to the
         ``autodoc-skip-member`` event.
         """
         ret = []
         isattr = False
         for (membername, member) in members:
-            ret.append((membername, member, isattr))
-
+            if not want_all:
+                ret.append((membername, member, isattr))
+            else:
+                if member.is_public() or self.options.private_members:
+                    ret.append((membername, member, isattr))
         return ret
 
     def format_name(self) -> str:
@@ -245,16 +324,6 @@ class CObjectDocumenter(Documenter):
         """
         return self.object.format_args(**kwargs)
 
-    def add_directive_header(self, sig: str) -> None:
-        """Add the directive header and options to the generated content."""
-
-        # save off the :attr:`objpath`, this will prevent the :module: option
-        # from being populated in some directives
-        objpath = self.objpath
-        self.objpath = []
-        super().add_directive_header(sig)
-        self.objpath = objpath
-
 
 class CModuleDocumenter(CObjectDocumenter):
     """
@@ -444,7 +513,8 @@ class CTypeDocumenter(CObjectDocumenter):
 
         return directive
 
-    def _merge_directives(self, directives: List[StringList]) -> StringList:
+    @staticmethod
+    def _merge_directives(directives: List[StringList]) -> StringList:
         """
         Args:
             directives (list(StringList)): The list of directives to merge.
@@ -454,15 +524,24 @@ class CTypeDocumenter(CObjectDocumenter):
         """
         merged_heading = StringList()
         merged_directive = StringList()
+        merged_options = StringList()
         for directive in directives:
+            options, _, _ = directive.get_indented(
+                1, until_blank=True, strip_indent=False
+            )
+            if options:
+                merged_options.extend(options)
+                del directive[1 : 1 + len(options)]
+
             directive_heading = directive[0]
-            directive[0] = self.indent
+            del directive[0]
 
             merged_directive.extend(directive)
             if len(directive_heading) > len(merged_heading):
                 merged_heading = directive_heading
 
-        merged_directive[0] = merged_heading
+        merged_directive.insert(0, merged_options)
+        merged_directive.insert(0, merged_heading, source=merged_directive.source(0))
         return merged_directive
 
     def consolidate_members(self) -> StringList:
@@ -573,7 +652,7 @@ class CDataDocumenter(CObjectDocumenter):
         return isinstance(parent, CObjectDocumenter) and member.type == "variable"
 
 
-class CModule(SphinxDirective):
+class CModule(CObject):
     """
     Module directive for C files
     """
@@ -611,3 +690,5 @@ def setup(app: Sphinx) -> None:
     app.add_directive_to_domain("c", "module", CModule)
     app.add_config_value("c_autodoc_roots", [""], "env")
     app.add_event("c-autodoc-pre-process")
+
+    patch_c_domain()
