@@ -9,12 +9,12 @@ import textwrap
 
 from collections import OrderedDict, namedtuple
 from itertools import takewhile
-from typing import Any, List, Optional, Union, Dict
+from typing import Any, List, Optional, Union, Dict, Tuple
 
 from bs4 import BeautifulSoup
 from bs4.element import Tag
 from clang import cindex
-from clang.cindex import Cursor, Token
+from clang.cindex import Cursor, Token, StorageClass
 
 from sphinx_c_autodoc.clang.patches import patch_clang
 
@@ -62,9 +62,11 @@ class DocumentedObject:
             example for structs this would be the members or fields.
         _soup (:class:`~bs4.BeautifulSoup`): The soupified version of
             :attr:`node`'s clang xml comment.
-        _declaration: The declaration string. For most things this is
+        _declaration (str): The declaration string. For most things this is
             the type as well as the name.
-
+        _line_range (Tuple[int, int]): The line range of the C construct,
+            this will include any leading or trailing comments that may be
+            part of the constructs documentation.
     """
 
     type_ = "object"
@@ -76,6 +78,27 @@ class DocumentedObject:
         self._children: Optional[OrderedDict] = None
         self._soup: Optional[BeautifulSoup] = None
         self._declaration: Optional[str] = None
+        self._line_range: Optional[Tuple[int, int]] = None
+
+    def line_range(self) -> Tuple[int, int]:
+        """
+        The lines in the source file that this object covers.
+
+        This will include any leading or trailing comments that may be part
+        of the constructs documentation.
+        """
+        if self._line_range is None:
+            node_extent = self.node.extent
+            comment_extent = self.node.comment_extent
+            if comment_extent.start.file is None:
+                comment_extent = node_extent
+
+            self._line_range = (
+                min(node_extent.start.line, comment_extent.start.line),
+                max(node_extent.end.line, comment_extent.end.line),
+            )
+
+        return self._line_range
 
     @property
     def type(self) -> str:
@@ -118,6 +141,11 @@ class DocumentedObject:
         obj_dict["doc"] = self.doc
         obj_dict["type"] = self.type
         obj_dict["name"] = self.name
+
+        line_range = self.line_range()
+        obj_dict["start_line"] = line_range[0]
+        obj_dict["end_line"] = line_range[1]
+
         if self.children:
             obj_dict["children"] = []
 
@@ -214,6 +242,29 @@ class DocumentedObject:
         paragraph += "\n"
         return paragraph
 
+    def is_public(self) -> bool:
+        """
+        Determines if this item is public.
+
+        C doesn't actually have public/private namespace so we're going to
+        make up some rules.
+
+        Constructs are public if:
+
+            - They are in a header file. By nature header files are meant to
+              be included in other files, thus they are deamed public.
+
+            - They can be visible outside of the compilation unit. These are
+              things the linker can get access to. Mainly this means functions
+              and variables that are not static.
+        """
+        # Here we'll do the most common logic, and let specific constructs that
+        # can be public do special logic.
+        if self.node.location.file.name.endswith(".h"):
+            return True
+
+        return False
+
 
 class DocumentedFile(DocumentedObject):
     """
@@ -326,6 +377,13 @@ class DocumentedMember(DocumentedObject):
         type_ = self.node.type.spelling
         return f"{type_} {self.name}"
 
+    def is_public(self) -> bool:
+        """
+        Members are always public, because it's their parents that determine
+        public versus private.
+        """
+        return True
+
 
 class DocumentedFunction(DocumentedObject):
     """
@@ -419,6 +477,15 @@ class DocumentedFunction(DocumentedObject):
 
         return "{} {}({})".format(return_type, func.spelling, ", ".join(args))
 
+    def is_public(self) -> bool:
+        """
+        Functions are public as long as they are not static.
+        """
+        if self.node.storage_class == StorageClass.STATIC:
+            return False
+
+        return True
+
 
 class DocumentedType(DocumentedObject):
     """
@@ -489,7 +556,7 @@ class DocumentedUnion(DocumentedStructure):
 
 class DocumentedEnum(DocumentedStructure):
     """
-    Class for unions. Same as structures with a different :attr:`type`.
+    Class for Enumerations. Same as structures with a different :attr:`type`.
     """
 
     type_ = "enum"
@@ -519,6 +586,15 @@ class DocumentedVariable(DocumentedObject):
         name, _, _ = decl.partition("=")
         return name
 
+    def is_public(self) -> bool:
+        """
+        Variables are public as long as they are not static.
+        """
+        if self.node.storage_class == StorageClass.STATIC:
+            return False
+
+        return True
+
 
 CURSORKIND_TO_OBJECT_CLASS = {
     cindex.CursorKind.TRANSLATION_UNIT: DocumentedFile,
@@ -542,7 +618,11 @@ def object_from_cursor(cursor: Cursor) -> Optional[DocumentedObject]:
     name = cursor.spelling
 
     if not name:
-        if cursor.kind in ALLOWED_ANONYMOUS:
+        # An anonymous construct which isn't contained in a typedef will have a
+        # type spelling of:
+        # "<construct> (anonymous at # <path_to_c_file>:<lineno>)"
+        # Typedef's should be handled by the get_nested_node() function
+        if cursor.kind in ALLOWED_ANONYMOUS and "anonymous at" in cursor.type.spelling:
             filename = os.path.basename(cursor.location.file.name)
             # remove the extension from the filename since the '.' is not a
             # valid c identifier. splitext will remove the trailing most
@@ -708,10 +788,10 @@ def comment_nodes(cursor: Cursor, children: List[Cursor]) -> None:
             continue
 
         # This may not be 100% accurate but move the end to the previous
-        # line. This solves problems like macro defintions not including the
+        # line. This solves problems like macro definitions not including the
         # preprocessor `#define` tokens.
         #
-        #                             <-- prveious line
+        #                             <-- previous line
         #     #define SOME_MACRO 23
         #             ^            ^ (Note `end` is exclusive)
         #             |            |
